@@ -1,19 +1,18 @@
-from __future__ import print_function, division, absolute_import
-
 from collections import deque
 import logging
 
+import dask
 from tornado import gen, locks
 from tornado.ioloop import IOLoop
 
-from .config import config
 from .core import CommClosedError
+from .utils import parse_timedelta
 
 
 logger = logging.getLogger(__name__)
 
 
-class BatchedSend(object):
+class BatchedSend:
     """ Batch messages in batches on a stream
 
     This takes an IOStream and an interval (in ms) and ensures that we send no
@@ -25,8 +24,8 @@ class BatchedSend(object):
 
     Example
     -------
-    >>> stream = yield connect(ip, port)
-    >>> bstream = BatchedSend(interval=10)  # 10 ms
+    >>> stream = yield connect(address)
+    >>> bstream = BatchedSend(interval='10 ms')
     >>> bstream.start(stream)
     >>> bstream.send('Hello,')
     >>> bstream.send('world!')
@@ -35,13 +34,13 @@ class BatchedSend(object):
 
         ['Hello,', 'world!']
     """
+
     # XXX why doesn't BatchedSend follow either the IOStream or Comm API?
 
-    def __init__(self, interval, loop=None):
+    def __init__(self, interval, loop=None, serializers=None):
         # XXX is the loop arg useful?
         self.loop = loop or IOLoop.current()
-        self.interval = interval / 1000.
-
+        self.interval = parse_timedelta(interval, default="ms")
         self.waker = locks.Event()
         self.stopped = locks.Event()
         self.please_stop = False
@@ -51,14 +50,25 @@ class BatchedSend(object):
         self.batch_count = 0
         self.byte_count = 0
         self.next_deadline = None
-        self.recent_message_log = deque(maxlen=config.get('recent-messages-log-length', 0))
+        self.recent_message_log = deque(
+            maxlen=dask.config.get("distributed.comm.recent-messages-log-length")
+        )
+        self.serializers = serializers
 
     def start(self, comm):
         self.comm = comm
         self.loop.add_callback(self._background_send)
 
+    def closed(self):
+        return self.comm and self.comm.closed()
+
     def __repr__(self):
-        return '<BatchedSend: %d in buffer>' % len(self.buffer)
+        if self.closed():
+            return "<BatchedSend: closed>"
+        else:
+            return "<BatchedSend: %d in buffer>" % len(self.buffer)
+
+    __str__ = __repr__
 
     @gen.coroutine
     def _background_send(self):
@@ -72,19 +82,20 @@ class BatchedSend(object):
                 # Nothing to send
                 self.next_deadline = None
                 continue
-            if (self.next_deadline is not None and
-                    self.loop.time() < self.next_deadline):
+            if self.next_deadline is not None and self.loop.time() < self.next_deadline:
                 # Send interval not expired yet
                 continue
             payload, self.buffer = self.buffer, []
             self.batch_count += 1
             self.next_deadline = self.loop.time() + self.interval
             try:
-                nbytes = yield self.comm.write(payload)
+                nbytes = yield self.comm.write(
+                    payload, serializers=self.serializers, on_error="raise"
+                )
                 if nbytes < 1e6:
                     self.recent_message_log.append(payload)
                 else:
-                    self.recent_message_log.append('large-message')
+                    self.recent_message_log.append("large-message")
                 self.byte_count += nbytes
             except CommClosedError as e:
                 logger.info("Batched Comm Closed: %s", e)
@@ -112,18 +123,23 @@ class BatchedSend(object):
             self.waker.set()
 
     @gen.coroutine
-    def close(self):
-        """ Flush existing messages and then close comm """
+    def close(self, timeout=None):
+        """ Flush existing messages and then close comm
+
+        If set, raises `tornado.util.TimeoutError` after a timeout.
+        """
         if self.comm is None:
             return
         self.please_stop = True
         self.waker.set()
-        yield self.stopped.wait()
+        yield self.stopped.wait(timeout=timeout)
         if not self.comm.closed():
             try:
                 if self.buffer:
                     self.buffer, payload = [], self.buffer
-                    yield self.comm.write(payload)
+                    yield self.comm.write(
+                        payload, serializers=self.serializers, on_error="raise"
+                    )
             except CommClosedError:
                 pass
             yield self.comm.close()
