@@ -1,5 +1,4 @@
-from __future__ import print_function, division, absolute_import
-
+import asyncio
 from functools import partial
 import gc
 import subprocess
@@ -8,15 +7,21 @@ from time import sleep
 from threading import Lock
 import unittest
 import weakref
+from distutils.version import LooseVersion
 
 from tornado.ioloop import IOLoop
 from tornado import gen
+import tornado
 import pytest
 
-from distributed import Client, Worker, Nanny
+from dask.system import CPU_COUNT
+from distributed import Client, Worker, Nanny, get_client
 from distributed.deploy.local import LocalCluster, nprocesses_nthreads
 from distributed.metrics import time
-from distributed.utils_test import (
+from distributed.system import MEMORY_LIMIT
+from distributed.utils_test import (  # noqa: F401
+    clean,
+    cleanup,
     inc,
     gen_test,
     slowinc,
@@ -25,10 +30,10 @@ from distributed.utils_test import (
     assert_can_connect_from_everywhere_4,
     assert_can_connect_from_everywhere_4_6,
     captured_logger,
+    tls_only_security,
 )
 from distributed.utils_test import loop  # noqa: F401
-from distributed.utils import sync
-from distributed.worker import TOTAL_MEMORY
+from distributed.utils import sync, TimeoutError
 
 from distributed.deploy.utils_test import ClusterTest
 
@@ -46,7 +51,7 @@ def test_simple(loop):
             x = e.submit(inc, 1)
             x.result()
             assert x.key in c.scheduler.tasks
-            assert any(w.data == {x.key: 2} for w in c.workers)
+            assert any(w.data == {x.key: 2} for w in c.workers.values())
 
             assert e.loop is c.loop
 
@@ -87,10 +92,10 @@ def test_procs():
         silence_logs=False,
     ) as c:
         assert len(c.workers) == 2
-        assert all(isinstance(w, Worker) for w in c.workers)
+        assert all(isinstance(w, Worker) for w in c.workers.values())
         with Client(c.scheduler.address) as e:
-            assert all(w.ncores == 3 for w in c.workers)
-            assert all(isinstance(w, Worker) for w in c.workers)
+            assert all(w.nthreads == 3 for w in c.workers.values())
+            assert all(isinstance(w, Worker) for w in c.workers.values())
         repr(c)
 
     with LocalCluster(
@@ -102,12 +107,12 @@ def test_procs():
         silence_logs=False,
     ) as c:
         assert len(c.workers) == 2
-        assert all(isinstance(w, Nanny) for w in c.workers)
+        assert all(isinstance(w, Nanny) for w in c.workers.values())
         with Client(c.scheduler.address) as e:
-            assert all(v == 3 for v in e.ncores().values())
+            assert all(v == 3 for v in e.nthreads().values())
 
-            c.start_worker()
-            assert all(isinstance(w, Nanny) for w in c.workers)
+            c.scale(3)
+            assert all(isinstance(w, Nanny) for w in c.workers.values())
         repr(c)
 
 
@@ -128,7 +133,7 @@ def test_move_unserializable_data():
             assert y.result() is lock
 
 
-def test_transports():
+def test_transports_inproc():
     """
     Test the transport chosen by LocalCluster depending on arguments.
     """
@@ -140,6 +145,8 @@ def test_transports():
         with Client(c.scheduler.address) as e:
             assert e.submit(inc, 4).result() == 5
 
+
+def test_transports_tcp():
     # Have nannies => need TCP
     with LocalCluster(
         1, processes=True, silence_logs=False, dashboard_address=None
@@ -149,6 +156,8 @@ def test_transports():
         with Client(c.scheduler.address) as e:
             assert e.submit(inc, 4).result() == 5
 
+
+def test_transports_tcp_port():
     # Scheduler port specified => need TCP
     with LocalCluster(
         1,
@@ -167,7 +176,7 @@ def test_transports():
 @pytest.mark.skipif("sys.version_info[0] == 2", reason="")
 class LocalTest(ClusterTest, unittest.TestCase):
     Cluster = partial(LocalCluster, silence_logs=False, dashboard_address=None)
-    kwargs = {"dashboard_address": None}
+    kwargs = {"dashboard_address": None, "processes": False}
 
 
 @pytest.mark.skipif("sys.version_info[0] == 2", reason="")
@@ -176,7 +185,7 @@ def test_Client_with_local(loop):
         1, scheduler_port=0, silence_logs=False, dashboard_address=None, loop=loop
     ) as c:
         with Client(c) as e:
-            assert len(e.ncores()) == len(c.workers)
+            assert len(e.nthreads()) == len(c.workers)
             assert c.scheduler_address in repr(c)
 
 
@@ -190,11 +199,11 @@ def test_Client_solo(loop):
 def test_duplicate_clients():
     pytest.importorskip("bokeh")
     c1 = yield Client(processes=False, silence_logs=False, dashboard_address=9876)
-    with pytest.warns(Exception) as info:
+    with pytest.warns(Warning) as info:
         c2 = yield Client(processes=False, silence_logs=False, dashboard_address=9876)
 
-    assert "bokeh" in c1.cluster.scheduler.services
-    assert "bokeh" in c2.cluster.scheduler.services
+    assert "dashboard" in c1.cluster.scheduler.services
+    assert "dashboard" in c2.cluster.scheduler.services
 
     assert any(
         all(
@@ -204,93 +213,155 @@ def test_duplicate_clients():
         for msg in info.list
     )
     yield c1.close()
+    yield c2.close()
 
 
 def test_Client_kwargs(loop):
     with Client(loop=loop, processes=False, n_workers=2, silence_logs=False) as c:
         assert len(c.cluster.workers) == 2
-        assert all(isinstance(w, Worker) for w in c.cluster.workers)
+        assert all(isinstance(w, Worker) for w in c.cluster.workers.values())
     assert c.cluster.status == "closed"
 
 
+def test_Client_unused_kwargs_with_cluster(loop):
+    with LocalCluster() as cluster:
+        with pytest.raises(Exception) as argexcept:
+            c = Client(cluster, n_workers=2, dashboard_port=8000, silence_logs=None)
+        assert (
+            str(argexcept.value)
+            == "Unexpected keyword arguments: ['dashboard_port', 'n_workers', 'silence_logs']"
+        )
+
+
+def test_Client_unused_kwargs_with_address(loop):
+    with pytest.raises(Exception) as argexcept:
+        c = Client(
+            "127.0.0.1:8786", n_workers=2, dashboard_port=8000, silence_logs=None
+        )
+    assert (
+        str(argexcept.value)
+        == "Unexpected keyword arguments: ['dashboard_port', 'n_workers', 'silence_logs']"
+    )
+
+
 def test_Client_twice(loop):
     with Client(loop=loop, silence_logs=False, dashboard_address=None) as c:
         with Client(loop=loop, silence_logs=False, dashboard_address=None) as f:
             assert c.cluster.scheduler.port != f.cluster.scheduler.port
 
 
-@pytest.mark.skipif("sys.version_info[0] == 2", reason="fork issues")
-def test_defaults():
-    from distributed.worker import _ncores
-
-    with LocalCluster(
-        scheduler_port=0, silence_logs=False, dashboard_address=None
+@pytest.mark.asyncio
+async def test_client_constructor_with_temporary_security(cleanup):
+    pytest.importorskip("cryptography")
+    async with Client(
+        security=True, silence_logs=False, dashboard_address=None, asynchronous=True
     ) as c:
-        assert sum(w.ncores for w in c.workers) == _ncores
-        assert all(isinstance(w, Nanny) for w in c.workers)
+        assert c.cluster.scheduler_address.startswith("tls")
+        assert c.security == c.cluster.security
 
-    with LocalCluster(
-        processes=False, scheduler_port=0, silence_logs=False, dashboard_address=None
+
+@pytest.mark.asyncio
+async def test_defaults(cleanup):
+    async with LocalCluster(
+        scheduler_port=0, silence_logs=False, dashboard_address=None, asynchronous=True
     ) as c:
-        assert sum(w.ncores for w in c.workers) == _ncores
-        assert all(isinstance(w, Worker) for w in c.workers)
+        assert sum(w.nthreads for w in c.workers.values()) == CPU_COUNT
+        assert all(isinstance(w, Nanny) for w in c.workers.values())
+
+
+@pytest.mark.asyncio
+async def test_defaults_2(cleanup):
+    async with LocalCluster(
+        processes=False,
+        scheduler_port=0,
+        silence_logs=False,
+        dashboard_address=None,
+        asynchronous=True,
+    ) as c:
+        assert sum(w.nthreads for w in c.workers.values()) == CPU_COUNT
+        assert all(isinstance(w, Worker) for w in c.workers.values())
         assert len(c.workers) == 1
 
-    with LocalCluster(
-        n_workers=2, scheduler_port=0, silence_logs=False, dashboard_address=None
+
+@pytest.mark.asyncio
+async def test_defaults_3(cleanup):
+    async with LocalCluster(
+        n_workers=2,
+        scheduler_port=0,
+        silence_logs=False,
+        dashboard_address=None,
+        asynchronous=True,
     ) as c:
-        if _ncores % 2 == 0:
-            expected_total_threads = max(2, _ncores)
+        if CPU_COUNT % 2 == 0:
+            expected_total_threads = max(2, CPU_COUNT)
         else:
-            # n_workers not a divisor of _ncores => threads are overcommitted
-            expected_total_threads = max(2, _ncores + 1)
-        assert sum(w.ncores for w in c.workers) == expected_total_threads
+            # n_workers not a divisor of _nthreads => threads are overcommitted
+            expected_total_threads = max(2, CPU_COUNT + 1)
+        assert sum(w.nthreads for w in c.workers.values()) == expected_total_threads
 
-    with LocalCluster(
-        threads_per_worker=_ncores * 2,
+
+@pytest.mark.asyncio
+async def test_defaults_4(cleanup):
+    async with LocalCluster(
+        threads_per_worker=CPU_COUNT * 2,
         scheduler_port=0,
         silence_logs=False,
         dashboard_address=None,
+        asynchronous=True,
     ) as c:
         assert len(c.workers) == 1
 
-    with LocalCluster(
-        n_workers=_ncores * 2,
+
+@pytest.mark.asyncio
+async def test_defaults_5(cleanup):
+    async with LocalCluster(
+        n_workers=CPU_COUNT * 2,
         scheduler_port=0,
         silence_logs=False,
         dashboard_address=None,
+        asynchronous=True,
     ) as c:
-        assert all(w.ncores == 1 for w in c.workers)
-    with LocalCluster(
+        assert all(w.nthreads == 1 for w in c.workers.values())
+
+
+@pytest.mark.asyncio
+async def test_defaults_6(cleanup):
+    async with LocalCluster(
         threads_per_worker=2,
         n_workers=3,
         scheduler_port=0,
         silence_logs=False,
         dashboard_address=None,
+        asynchronous=True,
     ) as c:
         assert len(c.workers) == 3
-        assert all(w.ncores == 2 for w in c.workers)
+        assert all(w.nthreads == 2 for w in c.workers.values())
 
 
-def test_worker_params():
-    with LocalCluster(
+@pytest.mark.asyncio
+async def test_worker_params(cleanup):
+    async with LocalCluster(
+        processes=False,
         n_workers=2,
         scheduler_port=0,
         silence_logs=False,
         dashboard_address=None,
         memory_limit=500,
+        asynchronous=True,
     ) as c:
-        assert [w.memory_limit for w in c.workers] == [500] * 2
+        assert [w.memory_limit for w in c.workers.values()] == [500] * 2
 
 
-def test_memory_limit_none():
-    with LocalCluster(
+@pytest.mark.asyncio
+async def test_memory_limit_none(cleanup):
+    async with LocalCluster(
         n_workers=2,
         scheduler_port=0,
         silence_logs=False,
         processes=False,
         dashboard_address=None,
         memory_limit=None,
+        asynchronous=True,
     ) as c:
         w = c.workers[0]
         assert type(w.data) is dict
@@ -298,24 +369,28 @@ def test_memory_limit_none():
 
 
 def test_cleanup():
-    c = LocalCluster(2, scheduler_port=0, silence_logs=False, dashboard_address=None)
-    port = c.scheduler.port
-    c.close()
-    c2 = LocalCluster(
-        2, scheduler_port=port, silence_logs=False, dashboard_address=None
-    )
-    c.close()
+    with clean(threads=False):
+        c = LocalCluster(
+            2, scheduler_port=0, silence_logs=False, dashboard_address=None
+        )
+        port = c.scheduler.port
+        c.close()
+        c2 = LocalCluster(
+            2, scheduler_port=port, silence_logs=False, dashboard_address=None
+        )
+        c2.close()
 
 
 def test_repeated():
-    with LocalCluster(
-        0, scheduler_port=8448, silence_logs=False, dashboard_address=None
-    ) as c:
-        pass
-    with LocalCluster(
-        0, scheduler_port=8448, silence_logs=False, dashboard_address=None
-    ) as c:
-        pass
+    with clean(threads=False):
+        with LocalCluster(
+            0, scheduler_port=8448, silence_logs=False, dashboard_address=None
+        ) as c:
+            pass
+        with LocalCluster(
+            0, scheduler_port=8448, silence_logs=False, dashboard_address=None
+        ) as c:
+            pass
 
 
 @pytest.mark.parametrize("processes", [True, False])
@@ -330,7 +405,7 @@ def test_bokeh(loop, processes):
         processes=processes,
         dashboard_address=0,
     ) as c:
-        bokeh_port = c.scheduler.services["bokeh"].port
+        bokeh_port = c.scheduler.services["dashboard"].port
         url = "http://127.0.0.1:%d/status/" % bokeh_port
         start = time()
         while True:
@@ -350,46 +425,47 @@ def test_bokeh(loop, processes):
 @pytest.mark.skipif(sys.version_info < (3, 6), reason="Unknown")
 def test_blocks_until_full(loop):
     with Client(loop=loop) as c:
-        assert len(c.ncores()) > 0
+        assert len(c.nthreads()) > 0
 
 
-@gen_test()
-def test_scale_up_and_down():
-    loop = IOLoop.current()
-    cluster = yield LocalCluster(
+@pytest.mark.asyncio
+async def test_scale_up_and_down():
+    async with LocalCluster(
         0,
         scheduler_port=0,
         processes=False,
         silence_logs=False,
         dashboard_address=None,
-        loop=loop,
         asynchronous=True,
-    )
-    c = yield Client(cluster, asynchronous=True)
+    ) as cluster:
+        async with Client(cluster, asynchronous=True) as c:
 
-    assert not cluster.workers
+            assert not cluster.workers
 
-    yield cluster.scale_up(2)
-    assert len(cluster.workers) == 2
-    assert len(cluster.scheduler.ncores) == 2
+            cluster.scale(2)
+            await cluster
+            assert len(cluster.workers) == 2
+            assert len(cluster.scheduler.nthreads) == 2
 
-    addr = cluster.workers[0].address
-    yield cluster.scale_down([addr])
+            cluster.scale(1)
+            await cluster
 
-    assert len(cluster.workers) == 1
-    assert addr not in cluster.scheduler.ncores
-
-    yield c.close()
-    yield cluster.close()
+            assert len(cluster.workers) == 1
 
 
+@pytest.mark.xfail(
+    sys.version_info >= (3, 8) and LooseVersion(tornado.version) < "6.0.3",
+    reason="Known issue with Python 3.8 and Tornado < 6.0.3. See https://github.com/tornadoweb/tornado/pull/2683.",
+    strict=True,
+)
 def test_silent_startup():
     code = """if 1:
         from time import sleep
         from distributed import LocalCluster
 
-        with LocalCluster(1, dashboard_address=None, scheduler_port=0):
-            sleep(1.5)
+        if __name__ == "__main__":
+            with LocalCluster(1, dashboard_address=None, scheduler_port=0):
+                sleep(.1)
         """
 
     out = subprocess.check_output(
@@ -417,7 +493,7 @@ def test_remote_access(loop):
         scheduler_port=0,
         silence_logs=False,
         dashboard_address=None,
-        ip="",
+        host="",
         loop=loop,
     ) as c:
         sync(loop, assert_can_connect_from_everywhere_4_6, c.scheduler.port)
@@ -433,7 +509,7 @@ def test_memory(loop, n_workers):
         dashboard_address=None,
         loop=loop,
     ) as cluster:
-        assert sum(w.memory_limit for w in cluster.workers) <= TOTAL_MEMORY
+        assert sum(w.memory_limit for w in cluster.workers.values()) <= MEMORY_LIMIT
 
 
 @pytest.mark.parametrize("n_workers", [None, 3])
@@ -449,12 +525,12 @@ def test_memory_nanny(loop, n_workers):
         with Client(cluster.scheduler_address, loop=loop) as c:
             info = c.scheduler_info()
             assert (
-                sum(w["memory_limit"] for w in info["workers"].values()) <= TOTAL_MEMORY
+                sum(w["memory_limit"] for w in info["workers"].values()) <= MEMORY_LIMIT
             )
 
 
 def test_death_timeout_raises(loop):
-    with pytest.raises(gen.TimeoutError):
+    with pytest.raises(TimeoutError):
         with LocalCluster(
             scheduler_port=0,
             silence_logs=False,
@@ -463,6 +539,7 @@ def test_death_timeout_raises(loop):
             loop=loop,
         ) as cluster:
             pass
+    LocalCluster._instances.clear()  # ignore test hygiene checks
 
 
 @pytest.mark.skipif(sys.version_info < (3, 6), reason="Unknown")
@@ -474,19 +551,21 @@ def test_bokeh_kwargs(loop):
         silence_logs=False,
         loop=loop,
         dashboard_address=0,
-        service_kwargs={"bokeh": {"prefix": "/foo"}},
+        service_kwargs={"dashboard": {"prefix": "/foo"}},
     ) as c:
 
-        bs = c.scheduler.services["bokeh"]
+        bs = c.scheduler.services["dashboard"]
         assert bs.prefix == "/foo"
 
 
 def test_io_loop_periodic_callbacks(loop):
-    with LocalCluster(loop=loop, silence_logs=False) as cluster:
+    with LocalCluster(
+        loop=loop, port=0, dashboard_address=None, silence_logs=False
+    ) as cluster:
         assert cluster.scheduler.loop is loop
         for pc in cluster.scheduler.periodic_callbacks.values():
             assert pc.io_loop is loop
-        for worker in cluster.workers:
+        for worker in cluster.workers.values():
             for pc in worker.periodic_callbacks.values():
                 assert pc.io_loop is loop
 
@@ -515,6 +594,31 @@ def test_ipywidgets(loop):
         assert isinstance(box, ipywidgets.Widget)
 
 
+def test_no_ipywidgets(loop, monkeypatch):
+    from unittest.mock import MagicMock
+
+    mock_display = MagicMock()
+
+    monkeypatch.setitem(sys.modules, "ipywidgets", None)
+    monkeypatch.setitem(sys.modules, "IPython.display", mock_display)
+
+    with LocalCluster(
+        n_workers=0,
+        scheduler_port=0,
+        silence_logs=False,
+        loop=loop,
+        dashboard_address=False,
+        processes=False,
+    ) as cluster:
+        cluster._ipython_display_()
+        args, kwargs = mock_display.display.call_args
+        res = args[0]
+        assert kwargs == {"raw": True}
+        assert isinstance(res, dict)
+        assert "text/plain" in res
+        assert "text/html" in res
+
+
 def test_scale(loop):
     """ Directly calling scale both up and down works as expected """
     with LocalCluster(
@@ -610,24 +714,27 @@ def test_adapt_then_manual(loop):
                 assert time() < start + 5
 
 
-def test_local_tls(loop):
-    from distributed.utils_test import tls_only_security
-
-    security = tls_only_security()
+@pytest.mark.parametrize("temporary", [True, False])
+def test_local_tls(loop, temporary):
+    if temporary:
+        pytest.importorskip("cryptography")
+        security = True
+    else:
+        security = tls_only_security()
     with LocalCluster(
         n_workers=0,
         scheduler_port=8786,
         silence_logs=False,
         security=security,
         dashboard_address=False,
-        ip="tls://0.0.0.0",
+        host="tls://0.0.0.0",
         loop=loop,
     ) as c:
         sync(
             loop,
             assert_can_connect_from_everywhere_4,
             c.scheduler.port,
-            connection_args=security.get_connection_args("client"),
+            connection_args=c.security.get_connection_args("client"),
             protocol="tls",
             timeout=3,
         )
@@ -637,7 +744,7 @@ def test_local_tls(loop):
             loop,
             assert_cannot_connect,
             addr="tcp://127.0.0.1:%d" % c.scheduler.port,
-            connection_args=security.get_connection_args("client"),
+            connection_args=c.security.get_connection_args("client"),
             exception_class=RuntimeError,
         )
 
@@ -690,7 +797,7 @@ def test_local_tls_restart(loop):
         silence_logs=False,
         security=security,
         dashboard_address=False,
-        ip="tls://0.0.0.0",
+        host="tls://0.0.0.0",
         loop=loop,
     ) as c:
         with Client(c.scheduler.address, loop=loop, security=security) as client:
@@ -750,7 +857,7 @@ def test_protocol_tcp(loop):
 )
 def test_protocol_ip(loop):
     with LocalCluster(
-        ip="tcp://127.0.0.2", loop=loop, n_workers=0, processes=False
+        host="tcp://127.0.0.2", loop=loop, n_workers=0, processes=False
     ) as cluster:
         assert cluster.scheduler.address.startswith("tcp://127.0.0.2")
 
@@ -768,7 +875,7 @@ def test_worker_class_worker(loop):
         scheduler_port=0,
         dashboard_address=None,
     ) as cluster:
-        assert all(isinstance(w, MyWorker) for w in cluster.workers)
+        assert all(isinstance(w, MyWorker) for w in cluster.workers.values())
 
 
 def test_worker_class_nanny(loop):
@@ -782,8 +889,148 @@ def test_worker_class_nanny(loop):
         scheduler_port=0,
         dashboard_address=None,
     ) as cluster:
-        assert all(isinstance(w, MyNanny) for w in cluster.workers)
+        assert all(isinstance(w, MyNanny) for w in cluster.workers.values())
 
 
-if sys.version_info >= (3, 5):
-    from distributed.deploy.tests.py3_test_deploy import *  # noqa F401
+@pytest.mark.asyncio
+async def test_worker_class_nanny_async(cleanup):
+    class MyNanny(Nanny):
+        pass
+
+    async with LocalCluster(
+        n_workers=2,
+        worker_class=MyNanny,
+        scheduler_port=0,
+        dashboard_address=None,
+        asynchronous=True,
+    ) as cluster:
+        assert all(isinstance(w, MyNanny) for w in cluster.workers.values())
+
+
+def test_starts_up_sync(loop):
+    cluster = LocalCluster(
+        n_workers=2,
+        loop=loop,
+        processes=False,
+        scheduler_port=0,
+        dashboard_address=None,
+    )
+    try:
+        assert len(cluster.scheduler.workers) == 2
+    finally:
+        cluster.close()
+
+
+def test_dont_select_closed_worker():
+    # Make sure distributed does not try to reuse a client from a
+    # closed cluster (https://github.com/dask/distributed/issues/2840).
+    with clean(threads=False):
+        cluster = LocalCluster(n_workers=0)
+        c = Client(cluster)
+        cluster.scale(2)
+        assert c == get_client()
+
+        c.close()
+        cluster.close()
+
+        cluster2 = LocalCluster(n_workers=0)
+        c2 = Client(cluster2)
+        cluster2.scale(2)
+
+        current_client = get_client()
+        assert c2 == current_client
+
+        cluster2.close()
+        c2.close()
+
+
+def test_client_cluster_synchronous(loop):
+    with clean(threads=False):
+        with Client(loop=loop, processes=False) as c:
+            assert not c.asynchronous
+            assert not c.cluster.asynchronous
+
+
+@pytest.mark.asyncio
+async def test_scale_memory_cores(cleanup):
+    async with LocalCluster(
+        n_workers=0,
+        processes=False,
+        threads_per_worker=2,
+        memory_limit="2GB",
+        asynchronous=True,
+    ) as cluster:
+        cluster.scale(cores=4)
+        assert len(cluster.worker_spec) == 2
+
+        cluster.scale(memory="6GB")
+        assert len(cluster.worker_spec) == 3
+
+        cluster.scale(cores=1)
+        assert len(cluster.worker_spec) == 1
+
+        cluster.scale(memory="7GB")
+        assert len(cluster.worker_spec) == 4
+
+
+@pytest.mark.asyncio
+async def test_repr(cleanup):
+    async with LocalCluster(
+        n_workers=2,
+        processes=False,
+        threads_per_worker=2,
+        memory_limit="2GB",
+        asynchronous=True,
+    ) as cluster:
+        text = repr(cluster)
+        assert "workers=2" in text
+        assert cluster.scheduler_address in text
+        assert "cores=4" in text or "threads=4" in text
+        assert "GB" in text and "4" in text
+
+    async with LocalCluster(
+        n_workers=2, processes=False, memory_limit=None, asynchronous=True
+    ) as cluster:
+        assert "memory" not in repr(cluster)
+
+
+@pytest.mark.asyncio
+@pytest.mark.parametrize("temporary", [True, False])
+async def test_capture_security(cleanup, temporary):
+    if temporary:
+        pytest.importorskip("cryptography")
+        security = True
+    else:
+        security = tls_only_security()
+    async with LocalCluster(
+        n_workers=0,
+        silence_logs=False,
+        security=security,
+        asynchronous=True,
+        dashboard_address=False,
+        host="tls://0.0.0.0",
+    ) as cluster:
+        async with Client(cluster, asynchronous=True) as client:
+            assert client.security == cluster.security
+
+
+@pytest.mark.asyncio
+@pytest.mark.skipif(
+    sys.version_info < (3, 7), reason="asyncio.all_tasks not implemented"
+)
+async def test_no_danglng_asyncio_tasks(cleanup):
+    start = asyncio.all_tasks()
+    async with LocalCluster(asynchronous=True, processes=False):
+        await asyncio.sleep(0.01)
+
+    tasks = asyncio.all_tasks()
+    assert tasks == start
+
+
+@pytest.mark.asyncio
+async def test_async_with():
+    async with LocalCluster(processes=False, asynchronous=True) as cluster:
+        w = cluster.workers
+        assert w
+
+    assert not w
